/**
 * Semestální práce č. 4 z předmětu KIV/VSP
 * (c) 2006 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.jsim;

import java.io.IOException;

import org.apache.log4j.Appender;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Layout;
import org.apache.log4j.SimpleLayout;

import cz.zcu.fav.kiv.jsim.JSimException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSimulation;

/**
 * @author habi
 * 
 */
public class Application {
	static {
		// switching off the default JSim logging
		java.util.logging.Logger l1 = java.util.logging.Logger.getLogger(JSimProcess.class.getName());
		l1.setLevel(java.util.logging.Level.WARNING);

		java.util.logging.Logger l2 = java.util.logging.Logger.getLogger(JSimSimulation.class.getName());
		l2.setLevel(java.util.logging.Level.WARNING);
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			Layout layout = new SimpleLayout();
			Appender appender = new FileAppender(layout, "app.log");
			BasicConfigurator.configure(appender);
		} catch (IOException e1) {
			e1.printStackTrace();
		}

		Double lambda = null;
		Double mu = null;
		Integer numberOfRepeats = null;

		if (args.length == 0 || args.length > 3) {
			lambda = 4d;
			mu = 0.5d;
			numberOfRepeats = 10000;
			System.out.println("3 params required: numberOfRepeats lambda mu");
			System.out.println("Using default values");
			System.out.println(" numberOfRepeats = " + numberOfRepeats);
			System.out.println(" lambda = " + lambda);
			System.out.println(" mu = " + mu);
		} else {
			numberOfRepeats = Integer.parseInt(args[0]);
			lambda = Double.parseDouble(args[1]);
			mu = Double.parseDouble(args[2]);
		}

		SystemSimulation simulation = null;
		try {
			simulation = new SystemSimulation(numberOfRepeats, lambda, mu);

			simulation.run();

			Statistics s = simulation.getStatistics();
			System.out.println("Střední doba provozu: " + s.getWorkingIntervalsMean());
			System.out.println("Střední doba prostoje: " + s.getBrokenIntervalsMean());

		} catch (JSimException e) {
			e.printStackTrace();
		} finally {
			if (simulation != null) {
				simulation.shutdown();
			}
		}
	}

}
