/**
 * Semestální práce č. 4 z předmětu KIV/VSP
 * (c) 2006 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.jsim;

import org.apache.log4j.Logger;

import cz.zcu.fav.kiv.jsim.JSimInvalidParametersException;
import cz.zcu.fav.kiv.jsim.JSimProcess;
import cz.zcu.fav.kiv.jsim.JSimSecurityException;
import cz.zcu.fav.kiv.jsim.JSimSimulationAlreadyTerminatedException;
import cz.zcu.fav.kiv.jsim.JSimTooManyProcessesException;

/**
 * This is a simple module with two parameters of distribution - working time
 * and repairing time.
 * 
 * @author habi
 * 
 */
public class Module extends JSimProcess {

	static Logger log = Logger.getLogger(Module.class);

	private boolean working = true;

	private double lambda;

	private double mu;
	
	private SystemSimulation systemSimulation;

	/**
	 * Creates a new instance of Module
	 * 
	 * @param name
	 * @param simulation
	 * @throws JSimSimulationAlreadyTerminatedException
	 * @throws JSimInvalidParametersException
	 * @throws JSimTooManyProcessesException
	 */
	public Module(String name, SystemSimulation simulation, double lambda, double mu) throws JSimSimulationAlreadyTerminatedException,
			JSimInvalidParametersException, JSimTooManyProcessesException {
		super(name, simulation);
		this.systemSimulation = simulation;
		this.lambda = lambda;
		this.mu = mu;
	}

	public boolean isWorking() {
		return working;
	}

	public void setWorking(boolean working) {
		this.working = working;
	}

	@Override
	protected void life() {
		log.info("Creating distributions for module " + this.getName());
		Distribution failDistribution = new ExponentialDistribution(lambda);
		Distribution repairDistribution = new ExponentialDistribution(mu);

		try {
			while (true) {
				if (isWorking()) {
					double d = failDistribution.getRandomValue();
					log.info(this.getName() + " - is working for " + d + " s");
					hold(d);

					setWorking(false);
					log.info(this.getName() + " - is now broken");
				} else {
					double d = repairDistribution.getRandomValue();
					log.info(this.getName() + " - is beeing repaired for " + d + " s");
					
					systemSimulation.addToQueue(this);
					passivate();
					
					hold(d);

					setWorking(true);
					log.info(this.getName() + " - is ready now");
				}
			}
		} catch (JSimSecurityException e) {
			log.error(e);
			e.printStackTrace();
		} catch (JSimInvalidParametersException e) {
			log.error(e);
			e.printStackTrace();
		}
	}

}
