/**
 * Semestální práce č. 4 z předmětu KIV/VSP
 * (c) 2006 Ivan Habernal
 * e-mail: habernal@students.zcu.cz
 */
package cz.habi.vsp.jsim;

import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;

/**
 * Simple statistics
 * 
 * @author habi
 * 
 */
public class Statistics {
	static Logger log = Logger.getLogger(Statistics.class);

	private Set<Double> workingIntervals = new HashSet<Double>();

	private Set<Double> repairingIntervals = new HashSet<Double>();

	private boolean previousState = true;

	private double intervalStart = 0d;

	/**
	 * Adds a current simulation time and system state to the statistics
	 * 
	 * @param currentState
	 *            working/broken
	 * @param currentTime
	 *            simulation time
	 */
	public void addTime(boolean currentState, double currentTime) {
		if (currentState == previousState) {
			log.info("State unchanged");
			return;
		}

		Double interval = new Double(currentTime - intervalStart);
		// startig new interval and state
		intervalStart = currentTime;

		if (previousState) {
			workingIntervals.add(interval);
			log.info(interval + " (s) added to working intervals");
		} else {
			repairingIntervals.add(interval);
			log.info(interval + " (s) added to repairing intervals");
		}

		previousState = currentState;
	}

	/**
	 * Returns the mean time, when the system is working
	 * 
	 * @return time
	 */
	public Double getWorkingIntervalsMean() {
		double total = 0d;
		for (Double d : workingIntervals) {
			total += d;
		}

		double result = total / (double) workingIntervals.size();
		log.info("Returning working interval mean: " + result);

		return result;
	}

	/**
	 * Returns the mean time, when the system is broken
	 * 
	 * @return time
	 */
	public Double getBrokenIntervalsMean() {
		double total = 0d;
		for (Double d : repairingIntervals) {
			total += d;
		}

		double result = total / (double) repairingIntervals.size();
		log.info("Returning broken interval mean: " + result);

		return result;
	}
}
